@@ -11,39 +11,10 @@ import torch.multiprocessing as mp
 import nncore
 
 
-def _bind_cuda_device():
-    if not torch.cuda.is_available():
-        return
-    local_rank = int(os.environ['LOCAL_RANK'])
-    assert local_rank < torch.cuda.device_count()
-    torch.cuda.set_device(local_rank)
-
-
-def _init_dist_pytorch(backend, **kwargs):
-    _bind_cuda_device()
-    dist.init_process_group(backend, **kwargs)
-
-
-def _init_dist_slurm(backend, **kwargs):
-    nodes = os.environ.get('SLURM_NODELIST', os.environ['SLURM_JOB_NODELIST'])
-    master_addr = getoutput(
-        'scontrol show hostname {} | head -n1'.format(nodes))
-
-    os.environ.setdefault('MASTER_ADDR', master_addr)
-    os.environ.setdefault('MASTER_PORT', '29500')
-
-    os.environ['WORLD_SIZE'] = os.environ['SLURM_NTASKS']
-    os.environ['RANK'] = os.environ['SLURM_PROCID']
-    os.environ['LOCAL_RANK'] = os.environ['SLURM_LOCALID']
-
-    _bind_cuda_device()
-    dist.init_process_group(backend, **kwargs)
-
-
 def _get_default_device(group=None):
     backend = dist.get_backend(group)
-    assert backend in ('gloo', 'nccl')
-    device = torch.device('cpu' if backend == 'gloo' else 'cuda')
+    assert backend in ('nccl', 'gloo')
+    device = torch.device('cuda' if backend == 'nccl' else 'cpu')
     return device
 
 
@@ -63,34 +34,83 @@ def _pad_tensor(data_tensor, pad_size):
     return data_tensor
 
 
-def init_dist(launcher='pytorch', backend='gloo', **kwargs):
+def init_dist(launcher=None, backend='nccl', **kwargs):
     """
     Initialize a distributed process group.
 
     Args:
-        launcher (str, optional): Launcher for the process group. Currently
-            supported launchers include ``pytorch`` and ``slurm``, Default:
-            ``'pytorch'``.
+        launcher (str | None, optional): Launcher for the process group.
+            Expected values include ``'torch'``, ``'slurm'``, and ``None``. If
+            not specified, this method will try to determine the launcher
+            automatically. Default: ``None``.
         backend (:obj:`dist.Backend` | str, optional): The distribution
             backend to use. This field should be given as a :obj:`dist.Backend`
             object or a str which can be accessed via :obj:`dist.Backend`
             attributes. Depending on build-time configurations, valid values
-            are ``'gloo'`` and ``'nccl'``. If using multiple processes per
+            are ``'nccl'`` and ``'gloo'``. If using multiple processes per
             machine with ``nccl`` backend, each process must have exclusive
             access to every GPU it uses, as sharing GPUs between processes can
-            result in deadlocks. Default: ``'gloo'``.
+            result in deadlocks. Default: ``'nccl'``.
+
+    Returns:
+        str | None: The launcher and backend info.
     """
-    assert backend in ('gloo', 'nccl')
+    assert launcher in ('torch', 'slurm', None)
+    assert backend in ('nccl', 'gloo')
+
+    if launcher is None:
+        if int(os.getenv('SLURM_NTASKS', 0)) > 1:
+            launcher = 'slurm'
+        elif int(os.getenv('WORLD_SIZE', 0)) > 1:
+            launcher = 'torch'
+        else:
+            return
 
     if mp.get_start_method(allow_none=True) is None:
         mp.set_start_method('spawn')
 
-    if launcher == 'pytorch':
-        _init_dist_pytorch(backend, **kwargs)
-    elif launcher == 'slurm':
-        _init_dist_slurm(backend, **kwargs)
+    if launcher == 'slurm':
+        node_list = os.getenv('SLURM_JOB_NODELIST',
+                              os.environ['SLURM_NODELIST'])
+        master_addr = getoutput(
+            'scontrol show hostname {} | head -n1'.format(node_list))
+
+        os.environ.setdefault('MASTER_ADDR', master_addr)
+        os.environ.setdefault('MASTER_PORT', '29500')
+
+        os.environ['WORLD_SIZE'] = os.environ['SLURM_NTASKS']
+        os.environ['RANK'] = os.environ['SLURM_PROCID']
+        os.environ['LOCAL_RANK'] = os.environ['SLURM_LOCALID']
+
+    if torch.cuda.is_available():
+        torch.cuda.set_device(int(os.environ['LOCAL_RANK']))
     else:
-        raise TypeError("unsupported launcher: '{}'".format(launcher))
+        backend = 'gloo'
+
+    dist.init_process_group(backend, **kwargs)
+    info = '{} ({})'.format(launcher, backend)
+
+    return info
+
+
+def is_elastic():
+    """
+    Check whether the current process was launched with ``dist.elastic``.
+
+    Returns:
+        bool: Whether the current process was launched with ``dist.elastic``.
+    """
+    return os.getenv('TORCHELASTIC_RUN_ID') is not None
+
+
+def is_slurm():
+    """
+    Check whether the current process was launched with Slurm.
+
+    Returns:
+        bool: Whether the current process was launched with Slurm.
+    """
+    return os.getenv('SLURM_NTASKS') is not None
 
 
 def is_distributed():
