@@ -4,21 +4,25 @@ import inspect
 from functools import wraps
 
 import h5py
-import numpy as np
+import jsonlines
 
 import nncore
-from .handlers import (Hdf5Handler, JsonHandler, PickleHandler, XmlHandler,
-                       YamlHandler)
+from .handlers import (HDF5Handler, JSONHandler, JSONLHandler, NumPyHandler,
+                       PickleHandler, TXTHandler, XMLHandler, YAMLHandler)
 
 _FILE_HANDLERS = {
-    'json': JsonHandler(),
-    'xml': XmlHandler(),
-    'yaml': YamlHandler(),
-    'yml': YamlHandler(),
+    'json': JSONHandler(),
+    'jsonl': JSONLHandler(),
+    'yaml': YAMLHandler(),
+    'yml': YAMLHandler(),
     'pickle': PickleHandler(),
     'pkl': PickleHandler(),
-    'hdf5': Hdf5Handler(),
-    'h5': Hdf5Handler()
+    'hdf5': HDF5Handler(),
+    'h5': HDF5Handler(),
+    'npy': NumPyHandler(),
+    'npz': NumPyHandler(),
+    'xml': XMLHandler(),
+    'txt': TXTHandler()
 }
 
 _open = open
@@ -32,18 +36,23 @@ def _get_handler(format):
 
 def load(name_or_file, format=None, **kwargs):
     """
-    Load data from json/xml/yaml/pickle/hdf5 files.
+    Load data from files.
 
     Args:
-        name_or_file (str or file object): Path to the file or a file object.
+        name_or_file (list | str | file object): Paths to the files or file
+            objects.
         format (str, optional): Format of the file. If not specified, the file
             format will be inferred from the file extension. Currently
-            supported formats include ``json``, ``xml``, ``yaml/yml``,
-            ``pickle/pkl`` and ``hdf5/h5``. Default: ``None``.
+            supported formats include ``json/jsonl``, ``yaml/yml``,
+            ``pickle/pkl``, ``hdf5/h5``, ``npy/npz``, ``xml``, and ``txt``.
+            Default: ``None``.
 
     Returns:
         any: The loaded data.
     """
+    if isinstance(name_or_file, (list, tuple)):
+        return [load(n, format=format, **kwargs) for n in name_or_file]
+
     format = format or nncore.pure_ext(name_or_file)
     handler = _get_handler(format)
 
@@ -57,26 +66,22 @@ def load(name_or_file, format=None, **kwargs):
                 type(name_or_file)))
 
 
-def dump(obj, name_or_file, format=None, overwrite=False, **kwargs):
+def dump(obj, name_or_file, format=None, overwrite=True, **kwargs):
     """
-    Dump data to json/xml/yaml/pickle/hdf5 files.
+    Dump data to a file.
 
     Args:
         obj (any): The object to be dumped.
-        name_or_file (str or file object): Path to the file or a file object.
+        name_or_file (str | file object): Path to the file or a file object.
         format (str, optional): Format of the file. If not specified, the file
             format will be inferred from the file extension. Currently
-            supported formats include ``json``, ``xml``, ``yaml/yml``,
-            ``pickle/pkl`` and ``hdf5/h5``. Default: ``None``.
+            supported formats include ``json/jsonl``, ``yaml/yml``,
+            ``pickle/pkl``, ``hdf5/h5``, ``npy/npz``, ``xml``, and ``txt``.
+            Default: ``None``.
         overwrite (bool, optional): Whether to overwrite it if the file exists.
-            Default: ``False``.
+            Default: ``True``.
     """
     format = format or nncore.pure_ext(name_or_file)
-    if format in ('hdf5', 'h5') and not isinstance(obj, np.ndarray):
-        raise TypeError(
-            "obj must be an np.ndarray for hdf5 files, but got '{}'".format(
-                type(obj)))
-
     handler = _get_handler(format)
 
     if isinstance(name_or_file, str):
@@ -97,48 +102,54 @@ def dump(obj, name_or_file, format=None, overwrite=False, **kwargs):
 
 def loads(string, format='pickle', **kwargs):
     """
-    Load data from a json/xml/yaml/pickle style string.
+    Load data from strings.
 
     Args:
-        string (str or btyearray): String of the data.
+        string (list | str | btyearray): Strings of the data.
         format (str, optional): Format of the string. Currently supported
-            formats include ``json``, ``xml``, ``yaml/yml`` and ``pickle/pkl``.
-            Default: ``'pickle'``.
+            formats include ``json/jsonl``, ``yaml/yml``, ``pickle/pkl`` and
+            ``xml``. Default: ``'pickle'``.
 
     Returns:
         any: The loaded data.
     """
-    assert format not in ('hdf5', 'h5')
+    if isinstance(string, (list, tuple)):
+        return [loads(s, format=format, **kwargs) for s in string]
+
     handler = _get_handler(format)
     return handler.load_from_str(string, **kwargs)
 
 
 def dumps(obj, format='pickle', **kwargs):
     """
-    Dump data to a json/xml/yaml/pickle style string.
+    Dump data to a string.
 
     Args:
         obj (any): The object to be dumped.
         format (str, optional): Format of the string. Currently supported
-            formats include ``json``, ``xml``, ``yaml/yml`` and ``pickle/pkl``.
-            Default: ``'pickle'``.
+            formats include ``json/jsonl``, ``yaml/yml``, ``pickle/pkl`` and
+            ``xml``. Default: ``'pickle'``.
 
     Returns:
         str: The dumped string.
     """
-    assert format not in ('hdf5', 'h5')
     handler = _get_handler(format)
     return handler.dump_to_str(obj, **kwargs)
 
 
-def list_from_file(filename, offset=0, separator=',', max_length=-1):
+def list_from_file(filename,
+                   encoding='utf-8',
+                   offset=0,
+                   separator=',',
+                   max_length=-1):
     """
     Load a text file and parse the content as a list of tuples or str.
 
     Args:
         filename (str): Path to the file to be loaded.
+        encoding (str, optional): The encoding of the file.
         offset (int, optional): The offset of line numbers. Default: ``0``.
-        separator (str or None, optional): The separator to use for parsing
+        separator (str | None, optional): The separator to use for parsing
             tuples. If not specified, each line would be treated as a str.
             Default: ``','``.
         max_length (int, optional): The maximum number of lines to be loaded.
@@ -149,7 +160,7 @@ def list_from_file(filename, offset=0, separator=',', max_length=-1):
         list[str]: The loaded str list.
     """
     out_list, count = [], 0
-    with _open(filename, 'r') as f:
+    with _open(filename, 'r', encoding=encoding) as f:
         for _ in range(offset):
             f.readline()
         for line in f:
@@ -172,25 +183,27 @@ def open(file=None, mode='r', format=None, as_decorator=None, **kwargs):
     Argument ``file`` and ``format`` should not be ``None`` at the same time.
 
     Args:
-        file (str or None, optional): Path to the file to be loaded.
+        file (str | None, optional): Path to the file to be loaded.
         mode (str, optional): The loading mode to use. Default: ``'r'``.
         format (str, optional): Format of the file. If not specified, the file
             format will be inferred from the file extension. Currently
-            supported formats include ``json``, ``yaml/yml``, ``pickle/pkl``
-            and ``hdf5/h5``. Default: ``None``.
-        as_decorator (bool or None, optional): Whether this method is used as
-            a decorator. Please explicitly assign a bool value to this
-            argument when using this method in a Python Shell. If not
-            specified, the method will try to determine it automatically.
+            supported formats include ``jsonl`` and ``hdf5/h5``. Default:
+            ``None``.
+        as_decorator (bool | None, optional): Whether this method is used as a
+            decorator. Please explicitly assign a bool value to this argument
+            when using this method in a Python Shell. If not specified, the
+            method will try to determine it automatically.
 
     Returns:
-        file-like object: The opened file object.
+        file object: The opened file object.
     """
     assert file is not None or format is not None
     format = format or nncore.pure_ext(file)
 
     if format in ('hdf5', 'h5'):
         handler = h5py.File
+    elif format == 'jsonl':
+        handler = jsonlines.open
     else:
         handler = _open
 
