@@ -9,6 +9,8 @@ from tempfile import TemporaryDirectory
 
 import nncore
 from .binder import bind_getter
+from .data import is_seq_of
+from .path import abs_path, cp, dir_name, is_file, join, pure_ext
 
 
 class CfgNode(OrderedDict):
@@ -104,7 +106,8 @@ class CfgNode(OrderedDict):
 
     def _check_freeze_state(self):
         if self._frozen:
-            raise RuntimeError('can not modify a frozen CfgNode object')
+            raise RuntimeError('can not modify a frozen {} object'.format(
+                self.__class__))
 
     def freeze(self):
         self._set_freeze_state(self, True)
@@ -130,27 +133,98 @@ class CfgNode(OrderedDict):
                 self[key].update(value)
 
     def merge_from(self, other):
+
+        def _insert(ori, value, cfg):
+            assert isinstance(cfg, (dict, int))
+            if isinstance(cfg, dict):
+                assert is_seq_of(cfg['index'], int) or isinstance(
+                    cfg['index'], int)
+                if isinstance(cfg['index'], (list, tuple)):
+                    assert isinstance(cfg['value'], (list, tuple)) and len(
+                        cfg['index']) == len(cfg['value'])
+                    idxs, vals = cfg['index'], cfg['value']
+                else:
+                    idxs, vals = [cfg['index']], [cfg['value']]
+                for i, idx in enumerate(idxs):
+                    ori.insert(idx, vals[i])
+            elif type(cfg) is int:
+                ori.insert(cfg, value)
+            else:
+                ori.append(value)
+            return ori
+
+        def _update(ori, value, cfg):
+            assert isinstance(cfg, (dict, int))
+            if isinstance(cfg, dict):
+                assert is_seq_of(cfg['index'], int) or isinstance(
+                    cfg['index'], int)
+                if isinstance(cfg['index'], (list, tuple)):
+                    assert isinstance(cfg['value'], (list, tuple)) and len(
+                        cfg['index']) == len(cfg['value'])
+                    idxs, vals = cfg['index'], cfg['value']
+                else:
+                    idxs, vals = [cfg['index']], [cfg['value']]
+                for i, idx in enumerate(idxs):
+                    if isinstance(ori[idx], dict) and isinstance(
+                            vals[i], dict):
+                        ori[idx].merge_from(vals[i])
+                    else:
+                        ori[idx] = vals[i]
+            elif isinstance(ori[cfg], dict):
+                ori[cfg].merge_from(value)
+            else:
+                ori[cfg] = value
+            return ori
+
         assert isinstance(other, dict)
         other = self.__class__(**other)
 
         for key, value in other.items():
-            if isinstance(value, dict):
-                delete = value.pop('_delete_', 0)
-                append = value.pop('_append_', 0)
-                assert not delete or not append
+            if key.startswith('_'):
+                continue
 
-                if key in self and isinstance(
-                        self[key], dict) and not delete and not append:
-                    self[key].merge_from(value)
-                elif key in self and append:
-                    if isinstance(self[key], list):
-                        self[key].append(value)
-                    elif isinstance(self[key], tuple):
-                        self[key] = tuple(list(self[key]) + [value])
-                    else:
-                        self[key] = [self[key]] + [value]
-                else:
-                    self[key] = value
+            if isinstance(value, dict):
+                refine = value.pop('_refine_', None)
+                delete = value.pop('_delete_', None)
+                insert = value.pop('_insert_', None)
+                update = value.pop('_update_', None)
+
+                do_insert = type(insert) is int or insert
+                do_update = type(update) is int or update
+
+                if key not in self and refine:
+                    continue
+
+                if key in self and isinstance(self[key], dict):
+                    assert not do_update and not (delete and do_insert)
+                    if do_insert:
+                        self[key] = _insert([self[key]], value, insert)
+                        continue
+                    elif not delete:
+                        self[key].merge_from(value)
+                        continue
+
+                if key in self and isinstance(self[key], (list, tuple)):
+                    if do_insert or do_update:
+                        is_tuple = isinstance(self[key], tuple)
+                        if is_tuple:
+                            self[key] = list(self[key])
+                        if do_insert:
+                            self[key] = _insert(self[key], value, insert)
+                        if do_update:
+                            self[key] = _update(self[key], value, update)
+                        if is_tuple:
+                            self[key] = tuple(self[key])
+                        continue
+
+                assert not do_insert and not do_update
+                tmp_node = self.__class__()
+                tmp_node.merge_from(value)
+                if tmp_node or not value:
+                    self[key] = tmp_node
+            elif value == '_delete_':
+                if key in self:
+                    del self[key]
             else:
                 self[key] = value
 
@@ -161,14 +235,14 @@ class CfgNode(OrderedDict):
                 base[key] = value.to_dict()
             elif isinstance(value, (list, tuple)):
                 base[key] = type(value)(
-                    o.to_dict() if isinstance(o, self.__class__) else o
-                    for o in value)
+                    v.to_dict() if isinstance(v, self.__class__) else v
+                    for v in value)
             else:
                 base[key] = value
         return base
 
-    def to_json(self):
-        return nncore.dumps(self.to_dict(), format='json', indent=2)
+    def to_json(self, indent=2):
+        return nncore.dumps(self.to_dict(), format='json', indent=indent)
 
 
 @bind_getter('filename')
@@ -188,21 +262,21 @@ class Config(CfgNode):
 
         Args:
             filename (str): Path to the config file. Currently supported
-                formats include ``py``, ``json`` and ``yaml/yml``.
+                formats include ``py``, ``json``, and ``yaml/yml``.
             freeze (bool, optional): Whether to freeze the config after
                 initialization.  Default: ``False``.
 
         Returns:
             :obj:`Config`: The constructed config object.
         """
-        filename = nncore.abs_path(filename)
-        nncore.is_file(filename, raise_error=True)
+        filename = abs_path(filename)
+        is_file(filename, raise_error=True)
 
-        format = nncore.pure_ext(filename)
+        format = pure_ext(filename)
         if format == 'py':
             with TemporaryDirectory() as tmp:
                 mod_name = str(int.from_bytes(os.urandom(2), 'big'))
-                nncore.cp(filename, nncore.join(tmp, '{}.py'.format(mod_name)))
+                cp(filename, join(tmp, '{}.py'.format(mod_name)))
                 sys.path.insert(0, tmp)
                 mod = import_module(mod_name)
                 sys.path.pop(0)
@@ -211,7 +285,7 @@ class Config(CfgNode):
                     for k, v in mod.__dict__.items()
                     if not k.startswith('__') or not k.endswith('__')
                 }
-        elif format in ['json', 'yml', 'yaml']:
+        elif format in ('json', 'yml', 'yaml'):
             cfg = nncore.load(filename)
         else:
             raise TypeError("unsupported format: '{}'".format(format))
@@ -223,8 +297,14 @@ class Config(CfgNode):
 
             _cfg = CfgNode()
             for name in base:
-                path = nncore.join(nncore.dir_name(filename), name)
-                _cfg.merge_from(Config.from_file(path))
+                if name.endswith(('.py', '.json', '.yml', '.yaml')):
+                    path = join(dir_name(filename), name)
+                    _cfg.merge_from(Config.from_file(path))
+                else:
+                    curr_path = os.getcwd()
+                    if curr_path not in sys.path:
+                        sys.path.insert(0, curr_path)
+                    import_module(name)
 
             _cfg.merge_from(cfg)
             cfg = _cfg
